/*
To Do:

disable buttons until disc is inserted
block input to parent windows when dialogs are up
set busy cursor for lengthy operations
fix text modification callbacks
spin widget

*/

/* phcd:  cd.c module                                    */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 1.1	 */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <time.h>
#include <signal.h>
#include <assert.h>
#include <sys/cdrom.h>
#include <sys/qioctl.h>
#ifdef __QNXNTO__
#include <sys/dcmd_blk.h>
#endif

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "phcd.h"
#include "abimport.h"
#include "proto.h"

cdinfo_t			cdinfo = { -1 };

int cd_check_discs( )
{
	disc_t *disc, *pdisc;
	int dnum;

	assert(cdinfo.mode == M_NODISC);
	
	if( !cd_disc_present( ) ) {
		cd_clear_discs( );
		cdinfo.disc_list = NULL;
//		cd_setsensitive( FALSE );
		return( 0 );
	}

	if( cdinfo.disc_list )
		cd_clear_discs( );

	cd_eject_load(1);
	
	// scan for all posible discs
	for( dnum = 1, pdisc = NULL; dnum <= CD_MAX_DISCS; dnum++ ) {
		if( ( disc = calloc( 1, sizeof( disc_t ) ) ) == NULL )
			break;
		disc->dnum = dnum;					// mark as valid entry
		disc->nxt = NULL;
		if( cdinfo.disc_list == NULL )
			cdinfo.disc_list = disc;
		if( !cd_change_disc( dnum ) ) {
			if( cdinfo.disc_list == disc )
				cdinfo.disc_list = NULL;
			free( disc );
			continue;
		}
		if( pdisc )
			pdisc->nxt = disc;
		pdisc = disc;
		
		switch (cd_read_toc( ) ) {
		case 1: break;
		case 0: free( disc );
			continue;
		case -1: {
		 		cdinfo.mode = M_DATA;
		 		display_DATA();
		 		return (0);
			}
		}
	}
	cdinfo.atrack = -1;
	cdinfo.mode = M_STOP;
	display_RESET();
//	cd_setsensitive( TRUE );
	return( 1 );
}

int cd_read_subchnl( struct cdrom_subchnl *sc )
{
	sc->cdsc_format = CDROM_MSF | ( 1 << 4 );
#ifdef __QNXNTO__
	if( devctl( cdinfo.fd, DCMD_BLK_CDROMSUBCHNL, sc, sizeof( *sc ), 0 ) )
#else
	if( qnx_ioctl(cdinfo.fd, CDROMSUBCHNL, sc, sizeof *sc, sc, sizeof *sc ) == -1 )
#endif
		return 0;
	return 1;
}

//
//  cd_read_toc
//  Returns:  1  Music disk
//            0  Error
//			 -1  Data disk
//
int cd_read_toc( )
{
	int						track, rval = 1;
	disc_t					*disc = cdinfo.adisc;
	cdrom_tocentry_t		*te;

#ifdef __QNXNTO__
	if( !devctl( cdinfo.fd, DCMD_BLK_CDROMREADTOCHDR, &disc->tochdr, sizeof( disc->tochdr ), 0 ) )
#else
	if( qnx_ioctl( cdinfo.fd, CDROMREADTOCHDR, 0, 0, &disc->tochdr, sizeof( disc->tochdr ) ) != -1 )
#endif
	{
		if( ( disc->tocentry = calloc( disc->tochdr.cdth_trk1 - disc->tochdr.cdth_trk0 + 2, sizeof( cdrom_tocentry_t ) ) ) == NULL )
			return( 0 );
		te = disc->tocentry;
 		for( track = disc->tochdr.cdth_trk0; track < disc->tochdr.cdth_trk1 + 2; track++, te++ ) {
			te->cdte_track = track;
			if( track == disc->tochdr.cdth_trk1 + 1 )
				te->cdte_track = CDROM_LEADOUT;
			te->cdte_format = CDROM_LBA;
#ifdef __QNXNTO__
			if( devctl( cdinfo.fd, DCMD_BLK_CDROMREADTOCENTRY, te, sizeof( *te ), 0 ) )
#else
			if( qnx_ioctl( cdinfo.fd, CDROMREADTOCENTRY, te, sizeof( *te ), te, sizeof( *te ) ) == -1 )
#endif
				{
					rval = 0;
				}
			if (te->cdte_ctrl & CDROM_DATA_DISK) rval = -1;
		} //for
	}
	else {
		rval = 0;
	}
	cdinfo.tot_trks = disc->tochdr.cdth_trk1 - disc->tochdr.cdth_trk0 + 1;
	return( rval );
}

int cd_get_volume( )
{
	struct cdrom_volctrl	vctrl;

#ifdef __QNXNTO__
	if( devctl( cdinfo.fd, DCMD_BLK_CDROMREADVOL, &vctrl, sizeof( vctrl ), 0 ) )
#else
	if(ioctl(cdinfo.fd, CDROMREADVOL, &vctrl) == (ushort_t)-1)
#endif
	{
		return 0;
	}
//	cdinfo.level = vctrl.channel0;
	return 0;
}

int cd_set_volume( )
{
	struct cdrom_volctrl	vctrl;

//	vctrl.channel0 = cdinfo.level_left;
//	vctrl.channel1 = cdinfo.level_right;
//	vctrl.channel2 = cdinfo.level_left;
//	vctrl.channel3 = cdinfo.level_right;
#ifdef __QNXNTO__
	if( devctl( cdinfo.fd, DCMD_BLK_CDROMVOLCTRL, &vctrl, sizeof( vctrl ), 0 ) )
#else
	if(ioctl(cdinfo.fd, CDROMVOLCTRL, &vctrl) == (ushort_t)-1)
#endif
	{
		perror("volume increase");
	}
	return 0;
}

int cd_disc_present( )
{
	struct cdrom_discinfo di;
int ret; 
#ifdef __QNXNTO__
	if( devctl( cdinfo.fd, DCMD_BLK_CDROMDISCINFO, &di, sizeof( di ), 0 ) )
#else
//	if(qnx_ioctl( cdinfo.fd, CDROMDISCINFO, &di, sizeof di, &di, sizeof di) == -1)
//     This function sometimes returns 0xffffffff  (invalid connection)
//     and sometimes returns           0xffff      (invalid disk)
//
//	   printf("ret = %x\n",ret);
	ret = qnx_ioctl( cdinfo.fd, CDROMDISCINFO, &di, sizeof di, &di, sizeof di);

	if((ushort_t) ret == (ushort_t) -1 )
#endif
	{
		di.flags = CDROM_DISCINFO_NODISC;
	}
	return( ( di.flags & CDROM_DISCINFO_NODISC ) ? 0 : 1 );
}

void cd_pause_resume( )
{
	if( cdinfo.mode == M_PAUSE ) {
#ifdef __QNXNTO__
		if( devctl( cdinfo.fd, DCMD_BLK_CDROMRESUME, NULL, 0, 0 ) )
#else
		if( ioctl(cdinfo.fd, CDROMRESUME) == (ushort_t)-1 )
#endif
		{
			printf( "resume failed\n");
			cdinfo.mode = M_NODISC;
			display_NODISC();
			return;
		}
		cdinfo.mode = M_PLAY;
	}
	else {
#ifdef __QNXNTO__
		if( devctl( cdinfo.fd, DCMD_BLK_CDROMPAUSE, NULL, 0, 0 ) )
#else
		if( ioctl( cdinfo.fd, CDROMPAUSE ) == (ushort_t)-1 )
#endif
		{
			printf( "pause failed\n");
			cdinfo.mode = M_NODISC;
			display_NODISC();
			return;
		}
		cdinfo.mode = M_PAUSE;
	}
}

void cd_stop( )
{

#ifdef __QNXNTO__
	if( devctl( cdinfo.fd, DCMD_BLK_CDROMSTOP, NULL, 0, 0 ) )
#else
	if( ioctl( cdinfo.fd, CDROMSTOP ) == (ushort_t)-1 )
#endif
	{
		printf( "stop failed\n");
		cdinfo.mode = M_NODISC;
		display_NODISC();
		return;
	}
	cdinfo.mode = M_STOP;
	cdinfo.reladdr.msf.minute = 0;
	cdinfo.reladdr.msf.second = 0;
	cd_update_time_display();

}

void cd_eject_load(int mode)
{
	static short first_time = 1;
	static short ejected;

	if (first_time)	{
		ejected =  !(cdinfo.mode == M_NODISC );
		first_time = 0;
	}

	if (mode != -1) {
		ejected = mode;
	}
	else if( ejected = !ejected ) {

		display_NULL();
		PtFlush();

#ifdef __QNXNTO__
		if( devctl( cdinfo.fd, DCMD_BLK_CDROMLOAD, NULL, 0, 0 ) )
#else
		if( ioctl( cdinfo.fd, CDROMLOAD ) == (ushort_t)-1 )
#endif
		{
			printf( "load failed\n");
			cdinfo.mode = M_NODISC;
			return;
		}
		display_NODISC();
		cdinfo.mode = M_NODISC;
		cd_check_discs();

	}
	else {
#ifdef __QNXNTO__
		if( devctl( cdinfo.fd, DCMD_BLK_CDROMEJECT, NULL, 0, 0 ) )
#else
		if( ioctl( cdinfo.fd, CDROMEJECT ) == (ushort_t)-1 )
#endif
		{
			printf( "eject failed\n");
			cdinfo.mode = M_NODISC;
			display_NODISC();
			return;
		}
		else {
			
			cdinfo.mode = M_NODISC;
			display_OPEN();
		}
	}
}


int cd_btns_cb( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )
{
	int			widget_name;

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	widget_name = ApName( widget );

	if( widget_name == ABN_cd_eject_btn ) {
		cd_eject_load(-1);
		return( Pt_CONTINUE );
	}
	else if( cdinfo.mode == M_NODISC ) return( Pt_CONTINUE );
	else if( cdinfo.mode == M_DATA ) return( Pt_CONTINUE );
	//
	// All other buttons assume that there is a valid CD-ROM disk
	//
	if( widget_name == ABN_cd_stop_btn ) {
			cd_stop( );
	}
	else if( widget_name == ABN_cd_play_btn ) {
		if( cdinfo.mode == M_PLAY )
			return( Pt_CONTINUE );
			
		// If positioned on a relative track, play from there.
		//
		if( cdinfo.atrack >= cdinfo.adisc->tochdr.cdth_trk0 &&
			cdinfo.atrack <= cdinfo.adisc->tochdr.cdth_trk1  )	cd_play( 0 );
		else 													cd_play( 1 );
	}
	else if( widget_name == ABN_cd_pause_btn ) {
		cd_pause_resume( );
	}
	else if( widget_name == ABN_cd_ntrk_btn ) {
		if( cdinfo.mode != M_PLAY ) {
			if (cdinfo.atrack < cdinfo.adisc->tochdr.cdth_trk1) {
				cdinfo.atrack ++;
				cd_update_track_display();
			}
			return( Pt_CONTINUE );
		}
		else cd_play( 1 );
	}
	else if( widget_name == ABN_cd_ptrk_btn ) {
		if( cdinfo.mode != M_PLAY ) {
			if (cdinfo.atrack > cdinfo.adisc->tochdr.cdth_trk0) {
				cdinfo.atrack --;
				cd_update_track_display();
			}
		}
		else cd_play( -1 );
	}

	return( Pt_CONTINUE );

}

int cd_pwr_btn_cb( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )
{
	PtArg_t	arg;

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	if( cdinfo.fd != -1 ) {
		cfg_dinit( );
		PtSetArg( &arg, Pt_ARG_FLAGS, 0, Pt_SET );
	}
	else {
		
		PtSetArg( &arg,
				  Pt_ARG_FLAGS,
					(cfg_init() ? 0 : Pt_TRUE ),
				  Pt_SET
				);
	}
	PtSetResources( widget, 1, &arg );
	return( Pt_END );
}



int cd_proxy_handler( PtWidget_t *wgt, void *data, PtCallbackInfo_t *cbinfo )
{
	wgt		= wgt;
	data	= data;
	cbinfo	= cbinfo;

	if (cdinfo.fd == -1) return(Pt_CONTINUE);
	switch( cdinfo.mode ) {
		case M_PLAY:
		case M_PAUSE:
			cd_play_status( );
			break;
		case M_STOP:
			break;
		case M_NODISC:
			cd_check_discs( );
			break;
		case M_DATA:
			break;
	}
	return( Pt_CONTINUE );
}

int cd_play_status( )
{
struct cdrom_subchnl sc;

if( !cd_read_subchnl( &sc ) ) {
// if the subchannel fails, check to see if the disc has been ejected
		return 0;
	}

	switch( sc.cdsc_audiostatus ) {
		case CDROM_AUDIO_PLAY:          /* audio play operation in progress */
//			cdinfo.atrack = sc.cdsc_trk;
			cdinfo.reladdr.msf = sc.cdsc_reladdr.msf;
			cdinfo.absaddr.msf = sc.cdsc_absaddr.msf;
			cd_update_display( );
		case CDROM_AUDIO_PAUSED:        /* audio play operation paused */
			break;
		case CDROM_AUDIO_ERROR:         /* audio play stopped due to error */
		// check for a disk...
		//	break;

		case CDROM_AUDIO_INVALID :		/* audio status not supported */
		case CDROM_AUDIO_NO_STATUS:     /* no current audio status to return */
			cdinfo.mode = M_NODISC;
			display_NODISC();
			break;
			
		case CDROM_AUDIO_COMPLETED:     /* audio play successfully completed */
			cdinfo.mode = M_STOP;
			cd_play( 1 );
			break;
	}
	return 0;
}


void cd_set_title(char *string)
{
	PtArg_t					args[1];

	PtSetArg( &args[0], Pt_ARG_WINDOW_TITLE, string, 0 );
	PtSetResources( ABW_base, 1, &args );
}


int cd_update_display( )
{

	if( cdinfo.iconified ) {
		char buff[50];

		sprintf( buff, "CD <%2.2d-%2.2d:%2.2d>", cdinfo.atrack, cdinfo.reladdr.msf.minute, cdinfo.reladdr.msf.second );
		cd_set_title(buff);
	}

	cd_update_time_display( );
	cd_update_track_display( );
	return 0;
}

int cd_wm_cb( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )
{
	PhWindowEvent_t		*ev = (PhWindowEvent_t *)cbinfo->cbdata;

	/* eliminate 'unreferenced' warnings */
	widget = widget, apinfo = apinfo, cbinfo = cbinfo;

	switch( ev->event_f ) {
		case Ph_WM_CLOSE:
			if( cdinfo.mode != M_STOP || cdinfo.mode != M_NODISC ) {
				cd_stop( );
			}
			break;
		case Ph_WM_ICON:
			cdinfo.iconified = !(ev->event_state);
			if (ev->event_state) cd_set_title("CD player");
			break;
		case Ph_WM_FOCUS:
// change taskbar palette gray
			break;
		default:
			break;
	}

	return( Pt_CONTINUE );
}

void cd_play_audio( long slba, long elba )
{
	struct cdrom_msf msf;

	// if we are playing, pause first
	if( cdinfo.mode == M_PLAY ) {
		cd_pause_resume( );
	}

	cdinfo.mode = M_PLAY;

	msf.cdmsf_min0		= LBA2MIN( slba );
	msf.cdmsf_sec0		= LBA2SEC( slba );
	msf.cdmsf_frame0	= LBA2FRAME( slba );
	msf.cdmsf_min1		= LBA2MIN( elba );
	msf.cdmsf_sec1		= LBA2SEC( elba );
	msf.cdmsf_frame1	= LBA2FRAME( elba );
#ifdef __QNXNTO__
	if( devctl( cdinfo.fd, DCMD_BLK_CDROMPLAYMSF, &msf, sizeof( msf ), 0 ) )
#else
	if( qnx_ioctl(cdinfo.fd, CDROMPLAYMSF, &msf, sizeof msf, 0, 0) == -1 )
#endif
	{
//		fprintf( stderr, "CDROMPLAYMSF failed, we are screwed\n");
		cdinfo.mode = M_NODISC;
		display_NODISC();
		return;
	}
}

void cd_update_track_display( )
{
	PtArg_t					args[2];
	int						idx;

	idx = cdinfo.atrack % 10;
	PtSetArg( &args[0], Pt_ARG_LABEL_DATA, imgs[idx], sizeof( PhImage_t ) );
	PtSetResources( ABW_track0_lbl, 1, &args );

	idx = cdinfo.atrack / 10;
	PtSetArg( &args[0], Pt_ARG_LABEL_DATA, imgs[idx], sizeof( PhImage_t ) );
	PtSetResources( ABW_track1_lbl, 1, &args );
}

void cd_update_time_display( )
{
	int					minutes, seconds, idx;
	cdrom_tocentry_t	*tocentry;
	PtArg_t				args[2];
	ulong_t				lba;

	minutes = seconds = 0;

	switch( cdinfo.tmode ) {
		case T_ELAPSED_TRACK:

			minutes = cdinfo.reladdr.msf.minute;
			seconds = cdinfo.reladdr.msf.second;
			break;
		case T_REMAIN_TRACK:
			lba = CD_TOCENTRY( cdinfo.atrack + 1 )->cdte_addr.lba - CD_TOCENTRY( cdinfo.atrack )->cdte_addr.lba;
			lba -= MSF2LBA( cdinfo.reladdr.msf.minute, cdinfo.reladdr.msf.second, cdinfo.reladdr.msf.frame );
			minutes = LBA2MIN( lba );
			seconds = LBA2SEC( lba );
			break;
		case T_REMAIN_DISC:
#ifdef FULL_PHCD
//			if( cdinfo.program || cdinfo.shuffle ) {
//				for( entry = cdinfo.apentry->nxt; entry; entry = entry->nxt ) {
//					tsec = LBA2MIN( entry->disc->tocentry[ entry->track + 1 - entry->disc->tochdr.cdth_trk0 ].cdte_addr.lba ) * 60 + LBA2SEC( entry->disc->tocentry[ entry->track + 1 - entry->disc->tochdr.cdth_trk0 ].cdte_addr.lba );
//					tsec -= LBA2MIN( entry->disc->tocentry[ entry->track - entry->disc->tochdr.cdth_trk0 ].cdte_addr.lba ) * 60 + LBA2SEC( entry->disc->tocentry[ entry->track - entry->disc->tochdr.cdth_trk0 ].cdte_addr.lba );
//					seconds += tsec;
//				}
//
//				tsec = LBA2MIN( CD_TOCENTRY( cdinfo.apentry->track + 1 )->cdte_addr.lba ) * 60 + LBA2SEC( CD_TOCENTRY( cdinfo.apentry->track + 1 )->cdte_addr.lba ) -
//						LBA2MIN( CD_TOCENTRY( cdinfo.apentry->track )->cdte_addr.lba ) * 60 + LBA2SEC( CD_TOCENTRY( cdinfo.apentry->track )->cdte_addr.lba );
//				tsec -= ( ( cdinfo.reladdr.msf.minute * 60 ) + cdinfo.reladdr.msf.second );
//
//				seconds += tsec;
//
//				minutes = seconds / 60;
//				seconds %= 60;
//			}
//			else
#endif
			{
				tocentry = CD_TOCENTRY( cdinfo.adisc->tochdr.cdth_trk1 - cdinfo.adisc->tochdr.cdth_trk0 + 2 );
				lba = tocentry->cdte_addr.lba - MSF2LBA( cdinfo.absaddr.msf.minute, cdinfo.absaddr.msf.second, cdinfo.reladdr.msf.frame );
				minutes = LBA2MIN( lba );
				seconds = LBA2SEC( lba );
			}
			break;
	}

	idx = seconds % 10;
	PtSetArg( &args[0], Pt_ARG_LABEL_DATA, imgs[idx], sizeof( PhImage_t ) );
	PtSetResources( ABW_sec0_lbl, 1, &args );

	idx = seconds / 10;
	PtSetArg( &args[0], Pt_ARG_LABEL_DATA, imgs[idx], sizeof( PhImage_t ) );
	PtSetResources( ABW_sec1_lbl, 1, &args );

	idx = minutes % 10;
	PtSetArg( &args[0], Pt_ARG_LABEL_DATA, imgs[idx], sizeof( PhImage_t ) );
	PtSetResources( ABW_min0_lbl, 1, &args );

	idx = minutes / 10;
	PtSetArg( &args[0], Pt_ARG_LABEL_DATA, imgs[idx], sizeof( PhImage_t ) );
	PtSetResources( ABW_min1_lbl, 1, &args );
}

void cd_play( int direction )
{
	ulong_t		slba, elba;

#ifdef FULL_PHCD
//	if( cdinfo.program || cdinfo.shuffle ) {	// play next program entry
//		if( cdinfo.apentry == NULL ) {			// start at the begining of the
//			if( cdinfo.shuffle ) {				// playlist
//				shuffle_playlist( );
//			}
//			cdinfo.apentry = cdinfo.plist;
//		}
//		else {
//			if( direction ) {
//				if( cdinfo.apentry->nxt == NULL && cdinfo.mode == M_PLAY ) {
//					return;
//				}
//				cdinfo.apentry = cdinfo.apentry->nxt;
//			}
//			else {
//				if( cdinfo.apentry->prev == NULL && cdinfo.mode == M_PLAY ) {
//					return;
//				}
//				cdinfo.apentry = cdinfo.apentry->prev;
//			}
//		}
//		if( cdinfo.apentry == NULL ) {
//			if( cdinfo.continuous ) {
//				if( cdinfo.shuffle ) {
//					shuffle_playlist( );
//				}
//				cdinfo.apentry = cdinfo.plist;
//			}
//			else {
//				return;
//			}
//		}
//		if( cdinfo.apentry->disc != cdinfo.adisc ) {	// new track isn't on active disc
//			cd_change_disc( cdinfo.apentry->disc->dnum );
//		}
//		cdinfo.atrack = cdinfo.apentry->track;
//	}
//	else
#endif
		{										// play next track
		if( cdinfo.atrack == -1 ) {
			cdinfo.atrack = cdinfo.adisc->tochdr.cdth_trk0;
		}
		else {
			cdinfo.atrack += direction;
			if( cdinfo.atrack < cdinfo.adisc->tochdr.cdth_trk0 &&
				cdinfo.atrack > cdinfo.adisc->tochdr.cdth_trk1 ) cdinfo.atrack = 0;
		}
		if( cdinfo.atrack > cdinfo.adisc->tochdr.cdth_trk1  ) {
			cdinfo.adisc = cdinfo.adisc->nxt;
			if( cdinfo.adisc == NULL ) {
				cdinfo.adisc = cdinfo.disc_list;
			}
			cd_change_disc( cdinfo.adisc->dnum );
			cdinfo.atrack = cdinfo.adisc->tochdr.cdth_trk0;
			if( !cdinfo.continuous ) {
				return;
			}
		}
	}
	slba = CD_TOCENTRY( cdinfo.atrack )->cdte_addr.lba;
	elba = CD_TOCENTRY( cdinfo.atrack + 1 )->cdte_addr.lba;
	if( cdinfo.intro ) {
		elba = slba + 1000;		// intro duration
	}
	cd_play_audio( slba, elba );
	cd_update_display();
}


int cd_advance_disc( )
{
	disc_t	*disc;

	disc = cdinfo.adisc->nxt;
	if( disc == NULL )		
		disc = cdinfo.disc_list;
		
	return( cd_change_disc( disc->dnum ) );
}

int cd_change_disc( int disc )
{
// fix this!
	cdinfo.adisc = cdinfo.disc_list;					// set active disc
//	update disc indicator;
	return( disc == 1 ? 1 : 0 );
}	

void cd_clear_discs( )
{
	disc_t *disc, *ndisc;

	for( disc = cdinfo.disc_list; disc; disc = ndisc ) {
		if( disc->tocentry )
			free( disc->tocentry );
		ndisc = disc->nxt;
		free( disc->dbase );
		free( disc );
	}
	cdinfo.disc_list = NULL;
	cdinfo.adisc = NULL;
}

void PtSetSensitive( PtWidget_t *widget, int action )
{
	PtArg_t arg;
	
	PtSetArg( &arg, Pt_ARG_FLAGS, action ? Pt_SELECTABLE : 0, Pt_SELECTABLE );
	PtSetResources( widget, 1, &arg );
}

void PtSetCursor( PtWidget_t *widget, short cursor )
{
	PtArg_t arg;
	
	PtSetArg( &arg, Pt_ARG_CURSOR_TYPE, cursor, 0 );
	PtSetResources( widget, 1, &arg );
}

void cd_setsensitive( int action )
{
//	PtSetSensitive( ABW_cd_program_btn, action );
//	PtSetSensitive( ABW_cd_srew_btn, action );
//	PtSetSensitive( ABW_cd_sfwd_btn, action );
	PtSetSensitive( ABW_cd_play_btn, action );
	PtSetSensitive( ABW_cd_stop_btn, action );
	PtSetSensitive( ABW_cd_pause_btn, action );
	PtSetSensitive( ABW_cd_ptrk_btn, action );
	PtSetSensitive( ABW_cd_ntrk_btn, action );
}

// callback for quick selection to a track, or entry in the play lisk
int cd_selector_cb( PtWidget_t *widget, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )
{
	int			widget_name, trk, etrk;
	disc_t		*disc = cdinfo.adisc;
	static int	extnd, strk;

	// Can't play anything if there is no disk...
	if (cdinfo.mode == M_NODISC) return ( Pt_CONTINUE );
	else if( cdinfo.mode == M_DATA ) return( Pt_CONTINUE );


	// can't do quick selection in shuffle mode
	if( cdinfo.shuffle )  return( Pt_CONTINUE );

	widget_name = ApName( widget );

	if( cdinfo.mode == M_PLAY ) {
		cd_pause_resume( );
	}

	if( widget_name == ABN_cd_sel0_button ) {
		trk = 0;
	}
	else if( widget_name == ABN_cd_sel1_button ) {
		trk = 1;
	}
	else if( widget_name == ABN_cd_sel2_button ) {
		trk = 2;
	}
	else if( widget_name == ABN_cd_sel3_button ) {
		trk = 3;
	}
	else if( widget_name == ABN_cd_sel4_button ) {
		trk = 4;
	}
	else if( widget_name == ABN_cd_sel5_button ) {
		trk = 5;
	}
	else if( widget_name == ABN_cd_sel6_button ) {
		trk = 6;
	}
	else if( widget_name == ABN_cd_sel7_button ) {
		trk = 7;
	}
	else if( widget_name == ABN_cd_sel8_button ) {
		trk = 8;
	}
	else if( widget_name == ABN_cd_sel9_button ) {
		trk = 9;
	}
	else if( widget_name == ABN_cd_sel10_button ) {
		extnd = 1;
		strk = 0;
			// clear index and time labels
		memset( &cdinfo.reladdr, 0, sizeof cdinfo.reladdr );
		cd_update_time_display(  );
		cdinfo.atrack = strk;
		{
		  PtArg_t args[1];
			PtSetArg( &args[0], Pt_ARG_LABEL_DATA, imgs[17], sizeof( PhImage_t ) );
			PtSetResources( ABW_track0_lbl, 1, &args );
			PtSetResources( ABW_track1_lbl, 1, &args );

		}
		PtFlush( );
		return( Pt_CONTINUE );
	}
	
	if( extnd ) {
		if (trk > (1 + cdinfo.adisc->tochdr.cdth_trk1 - cdinfo.adisc->tochdr.cdth_trk0)/10) return( Pt_CONTINUE );

		strk = trk * 10;
		extnd = 0;
cdinfo.atrack = strk;
		{
		  PtArg_t args[1];
			PtSetArg( &args[0], Pt_ARG_LABEL_DATA, imgs[trk], sizeof( PhImage_t ) );
			PtSetResources( ABW_track1_lbl, 1, &args );

		}
//		cd_update_track_display( );
	}
	else {
		if (strk + trk > (1 + cdinfo.adisc->tochdr.cdth_trk1 - cdinfo.adisc->tochdr.cdth_trk0)) return( Pt_CONTINUE );
		
		strk+= trk;
		cdinfo.atrack = strk;
		cd_update_track_display(  );
		memset( &cdinfo.reladdr, 0, sizeof cdinfo.reladdr );
		cd_update_time_display( );
		PtFlush( );
		etrk = strk + 1;
		if( etrk > disc->tochdr.cdth_trk1 )
			etrk = disc->tochdr.cdth_trk1 + 1;
		if( cdinfo.program ) {
// fix this
//			cd_play_audio( disc->tocentry[strk].cdte_addr.lba, disc->tocentry[etrk].cdte_addr.lba );
		}
		else {
// Fix this, don't allow strk above etrk
			cd_play_audio(CD_TOCENTRY(strk)->cdte_addr.lba,
						  CD_TOCENTRY(etrk)->cdte_addr.lba );
		}
		strk = 0;
	}

	/* eliminate 'unreferenced' warnings */
	apinfo = apinfo, cbinfo = cbinfo;
	
	return( Pt_CONTINUE );
}

