#ifndef __CD_H__
#define __CD_H__

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#ifndef __NEUTRINO__
#include <sys/ioctl.h>
#else
#include <sys/dcmd_blk.h>
#endif
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <dirent.h>
#include <pwd.h>
#include <string.h>
#include <sys/cdrom.h>

#ifndef FALSE
#define FALSE		0
#endif
#ifndef TRUE
#define TRUE		1
#endif

/* returns a pointer to the tocentry for the requested track */
#define CD_TOCENTRY( _trk )	(&cdinfo.adisc->tocentry[_trk - cdinfo.adisc->tochdr.cdth_trk0] )

/* Constant definitions */
#define MAXTRACK		100			/* Max number of tracks */
#define FRAME_PER_SEC	75			/* Frames per second */
#define MAX_VOL			100			/* Max logical audio volume */
#define CD_MAX_DISCS	1
#define OFFSET_UNKN		0xffffffff
#define OFFSET_THRESH	750

#define CDROM_DATA_DISK 0x04

/* Defines for the mode field in cdinfo_t */
#define M_NODISC	0			/* No disc loaded */
#define M_PLAY		1			/* Play mode */
#define M_PAUSE		2			/* Pause mode */
#define M_STOP		3			/* Stop mode */
#define M_DATA		4			/* Data disc loaded */

/* Defines for the time_dpy field in cdinfo_t */
#define T_ELAPSED_TRACK	0		/* Per-track elapsed time */
#define T_REMAIN_TRACK	1		/* Per-track remaining time */
#define T_REMAIN_DISC	2		/* Whole-disc remaining time */

#define FILE_PATH_SZ	72
#define STR_BUF_SZ		64
#define ERR_BUF_SZ		256

#define PHCD_VERSION	"0.01"
#define PHOTON_DIR		"/.photon"
#define PHCD_DIR		"/phcd"
#define PHCD_CFG		"/phcd.cfg"
#define CDPATH			"/dev/cd0"
#define MAX_DBDIRS		10
#define PGM_SEPCHAR		','

typedef struct cdrom_tochdr		cdrom_tochdr_t;
typedef struct cdrom_tocentry	cdrom_tocentry_t;
typedef struct dbase			dbase_t;
typedef struct link				link_t;
typedef struct disc				disc_t;
typedef struct program_entry	program_entry_t;
typedef struct cdinfo			cdinfo_t;
typedef union cdsc_addr			cdsc_addr_t;

union cdsc_addr {
	struct {
		unsigned char   minute;
		unsigned char   second;
		unsigned char   frame;
	} msf;
	long         lba;
};

struct dbase {
	ulong_t		discid;					/* Magic disc ID */
	char		ntrk;					/* Number of disc tracks */
	char		*dbfile;				/* Path to database file */
	char		category[255];			/* CD category */
	char		*dtitle;				/* Disc title */
	char		*artist;				/* Artist */
	char		*trklist[MAXTRACK];		/* Track title list */
	char		*extd;					/* Extended disc info */
	char		*extt[MAXTRACK];		/* Extended track info */
	char		*playorder;				/* Track play order */
};

struct link {
	char		*dtitle;				/* Disc title */
	char		idstr[9];				/* Disc id string */
	ulong_t		offset;					/* Distance measure */
	link_t		*next;					/* Link to next item */
};

struct disc {
	int						dnum;			/* disc number */
	cdrom_tochdr_t			tochdr;			/* toc hearder */
	cdrom_tocentry_t		*tocentry;		/* track information */
	dbase_t					*dbase;			/* database */
	disc_t					*nxt;
};

struct program_entry {
	disc_t					*disc;
	int						track;
	program_entry_t			*nxt;
	program_entry_t			*prev;
};

/* Structure containing current status information */
struct cdinfo {
	int				fd;						/* file descriptor */
	int				mode;					/* player mode */
	int				tmode;					/* time display mode */
	disc_t			*disc_list;				/* linked list of disc_t */
	disc_t			*adisc;					/* currently active disc */
	int				atrack;					/* active track */
	int				tot_trks;
	cdsc_addr_t		reladdr;				/* relative MSF */
	cdsc_addr_t		absaddr;				/* absolute MSF */
	program_entry_t	*plist;					/* program list */
	program_entry_t	*apentry;				/* active plist entry */
	int				continuous:1,			/* continuouse mode */
					shuffle:1,				/* shuffle mode */
					program:1,				/* program mode */
					intro:1,
					scan:1,					/* scan mode */
					lock:1,					/* caddy lock */
					iconified:1;			/* iconified */

	int				volume;					/* current volume level */
};

#endif
