/* phcd:  config.c module                                */
/*                            AppBuilder Photon Code Lib */
/*                                         Version 0.91B */

/* Standard headers */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <dirent.h>
#include <time.h>
#include <signal.h>
#include <sys/stat.h>
#include <sys/types.h>

/* Toolkit headers */
#include <Ph.h>
#include <Pt.h>
#include <Ap.h>

/* Local headers */
#include "phcd.h"
#include "abimport.h"
#include "proto.h"

int					cfg_changed = 0;
char				cdpath[256];
char				dbpath[256];
char				search_increment;
char				intro_duration;
char				*dbdirs[MAX_DBDIRS];
char				dbdirs_cnt;
ApDBase_t			*pictdb = NULL;
PhImage_t			*imgs[24];

static int start_song;

static void cfg_defaults(void)
{
	memset( &cdinfo, 0, sizeof( cdinfo ) );
	cdinfo.fd = -1;
	strcpy( cdpath, CDPATH );
	cdinfo.tmode = T_ELAPSED_TRACK;

	intro_duration = 10;
	search_increment = 5;
}


int cfg_open_device( )
{
	char	buf[255];

	if( cdinfo.fd != -1 ) {
		close( cdinfo.fd );
	}

	if( ( cdinfo.fd = open( cdpath, 0 ) ) == -1 ) {
//		sprintf( buf, "Unable To Open %s", cdpath );
//		PtAskQuestion( ABW_base, NULL, buf, "helv12b", "Ok", NULL, NULL, 1 );
		
		ApModuleParent(ABM_open_error, AB_PARENT, ABW_base);
		ApCreateModule(ABM_open_error, ABW_base, NULL);
		PtContainerGiveFocus(ABW_Error_Ack, NULL);
		return( -1 );
	}
	return( 0 );
}

void cfg_phabinit( )
{
	int						cnt;
	char					wname[10] = "spict_0";

	if( pictdb == NULL ) {

		pictdb = ApOpenDBase( ABM_displaypicts );
		
		for( cnt = 0; cnt < 10; cnt++, wname[6]++ ) {
			imgs[cnt] = ApGetImageRes( pictdb, wname );
		}
		imgs[cnt++] = ApGetImageRes( pictdb, "spict_ucolon" );	// 10
		imgs[cnt++] = ApGetImageRes( pictdb, "spict_colon" );	// 11
		imgs[cnt++] = ApGetImageRes( pictdb, "spict_space" );	// 12
		imgs[cnt++] = ApGetImageRes( pictdb, "spict_o" );		// 13
		imgs[cnt++] = ApGetImageRes( pictdb, "spict_p" );		// 14
		imgs[cnt++] = ApGetImageRes( pictdb, "spict_e" );		// 15
		imgs[cnt++] = ApGetImageRes( pictdb, "spict_n" );		// 16
		imgs[cnt++] = ApGetImageRes( pictdb, "spict_input" );	// 17
		imgs[cnt++] = ApGetImageRes( pictdb, "spict_d" );		// 18
		imgs[cnt++] = ApGetImageRes( pictdb, "spict_a" );		// 19
		imgs[cnt++] = ApGetImageRes( pictdb, "spict_t" );		// 20
		imgs[cnt++] = ApGetImageRes( pictdb, "spict_c" );		// 21
		imgs[cnt++] = ApGetImageRes( pictdb, "spict_i" );		// 22
		imgs[cnt++] = ApGetImageRes( pictdb, "spict_s" );		// 23
	}
	cfg_light_display( );
	return;
}



int cfg_init( )
{

	cfg_defaults();
	if( cfg_open_device( ) == -1 ) {
		return -1;
	}
	cfg_phabinit( );

	if( cd_check_discs( ) ) {
		// no disc, update display
	}
	return 0;
	
}

void cfg_dinit( )
{
	cd_stop( );
	cfg_dim_display( );
	cd_clear_discs( );
	close( cdinfo.fd );
	memset( &cdinfo, 0, sizeof( cdinfo ) );
	cdinfo.fd = -1;
}

void cfg_light_display( )
{
	PtArg_t					args[2];

	PtSetArg( &args[0], Pt_ARG_COLOR, Pg_CYAN, 0 );
	PtSetResources( ABW_sec_lbl, 1, &args );
	PtSetResources( ABW_min_lbl, 1, &args );
	PtSetResources( ABW_track_lbl, 1, &args );

	PtSetArg( &args[0], Pt_ARG_LABEL_DATA, imgs[11], sizeof( PhImage_t ) );
	PtSetResources( ABW_colon_lbl, 1, &args );

	display_NODISC();
}

void cfg_dim_display( )
{
	PtArg_t		args[1];

	PtSetArg( &args[0], Pt_ARG_COLOR, Pg_DBLUE, 0 );
	PtSetResources( ABW_sec_lbl, 1, args );
	PtSetResources( ABW_min_lbl, 1, args );
	PtSetResources( ABW_track_lbl, 1, args );

	PtSetArg( &args[0], Pt_ARG_LABEL_DATA, imgs[12], sizeof( PhImage_t ) );
	PtSetResources( ABW_sec0_lbl, 1, args );
	PtSetResources( ABW_sec1_lbl, 1, args );
	PtSetResources( ABW_min0_lbl, 1, args );
	PtSetResources( ABW_min1_lbl, 1, args );
	PtSetResources( ABW_track0_lbl, 1, args );
	PtSetResources( ABW_track1_lbl, 1, args );
	PtSetArg( &args[0], Pt_ARG_LABEL_DATA, imgs[10], sizeof( PhImage_t ) );
	PtSetResources( ABW_colon_lbl, 1, &args );
}


int
startup( PtWidget_t *link_instance, ApInfo_t *apinfo, PtCallbackInfo_t *cbinfo )

	{

	/* eliminate 'unreferenced' warnings */
	link_instance = link_instance, apinfo = apinfo, cbinfo = cbinfo;
	
	cfg_defaults( );
	if (start_song) {
		int power = !cfg_init();
		if (power) {
			PtArg_t	arg;
			PtSetArg( &arg, Pt_ARG_FLAGS, Pt_TRUE, Pt_SET);
			PtSetResources(ABW_cd_pwr_btn2, 1, &arg);
			cdinfo.atrack = start_song;
			cd_play(0);
		}
	}
	return( Pt_CONTINUE );

	}


int
parse_opts( int argc, char *argv[] )

	{

	/* eliminate 'unreferenced' warnings */
	argc = argc, argv = argv;
    if (argc > 2 &&
        argv[1][0] == 'a')  start_song = atoi(argv[2]);
    else start_song = 0;
    
	return( Pt_CONTINUE );

	}

